/*************************************************************************
Title:    simple wave scope with standard alpha numeric lcd modul.
Author:   Reiner Muench <muench@behnk.de>
Date:     October 1th 2002

Compiler: >->->->->->->->  GCCAVR  <-<-<-<-<-<-<-<-<-<-<-<-<

Hardware: AT90S8535/ 16 character LCD like Displaytech 161A
          29% of 8k AVR is used			

Function: The first 8 Digits are used for showing an actual recorded wave.
          The scope samples 40 bytes of analog signal, sync it and show 
		  it on LCD. A key on pin D3 wired to gnd select 8 ADC channels.
		  In the rest of the line the time of one periode and the actual 
		  channel is shown. The Trigger position is shown as vert. line. 
		  
		  The trick: A 40byte sample of analog signal are drawn as a curve
		  in 64byte of AVR memory. These Picture are transmitted as 8 user 
		  defined characters to LCD. Limitation is 40Bytes or 8 5x8chars. 
		  Try it and change it for your need.

		  The sample rate is 1ms or 1000sps. So a 50Hz sine wave is displayed
		  twice. It works also on higher rates. 
		  
**************************************************************************/

#include "include_all.h"	/* includes */

#define VTRG		127		/* trigger treshhold 0..255  */
#define TRGPOS		4		/* trigger position 0..39 */
#define PERIODE 	ON		/* periode or pulse measurment */
#define CURSOR1	 	ON		/* Trigger cursor on/off */
#define CURSOR2	 	ON		/* 2th cursor on/off */
#define HOLDOFF		200		/* holdoff time / display pause */		



#define TMR0RELOAD	255-(XTAL/1000/64)	/* Interrupt cycle time */

volatile signed int keytmr;

volatile u16		delaytmr;			// timer
volatile u16		runtime;			// timer
volatile u08		channel;	   	  	// actual ADC-Channel
volatile u08		graphbuf[64];	  	// Graphicbuffer
volatile u08		abuf[40];  		  	// Analog buffer
volatile u08		aptr;			  	// analogbuffer pointer
volatile u08		tptr;			  	// trigger pointer
volatile u08		einer, zehntel,time;
volatile u08		phase;				//

struct	bitfield {
		unsigned	bit_0 : 1,
					bit_1 : 1,
					bit_2 : 1,
					bit_3 : 1,
					bit_4 : 1,
					bit_5 : 1,
					bit_6 : 1,
					bit_7 : 1;
		};

volatile struct bitfield flag;
	
#define refresh flag.bit_0		
#define posedge flag.bit_1		
#define negedge flag.bit_2		
		
//#pragma interrupt_handler timer0:iv_TIMER0_OVF

//----------------------------------------------------------------------------
// 40x8 graphic display with a normal alphanumeric LCD
void drawgraph(void)
{
	u08 d, b;
	u08 bitmask;
	u08 adr;
	u08 ptr;
	u08 a;
	u08 t;
	
	for (d=0 ;d<64 ;d++)		// clear graph memory
		graphbuf[d] = 0;

#if(CURSOR1==ON)				// TRIGGER CURSOR on/off
	for (d=0 ;d<8 ;d++)			// set Cursor @ triggerpos
		graphbuf[8*(TRGPOS/5)+d] = ( 0x10>>(TRGPOS%5) );
#endif

	ptr = (39+tptr-TRGPOS)%40;	// aligment to 0..39 
	
	time = 0;
	t=0;
	posedge = 0;
	negedge = 0;
	
	for (d=0 ;d<8 ;d++)			// count 8 Digit
    	{
		bitmask = 0x10;			// start with ---10000
		
		for (b=0 ;b<5 ;b++)		// count 5 bits (bit765 not used in LCD)
	       	{
			a = abuf[ptr];
			adr = 7-(a>>5);
			graphbuf[8*d+adr] ^= bitmask;	// xor bits 
			
#if(PERIODE==ON)				/* PERIODE */
			if(!time)			// time measurement one periode
				{

				if(negedge)
					{
					t++;			// count time
					if(a>VTRG)
						posedge = 1;	// start pos. wave part
					}

				if(a<=VTRG)
					{
					negedge = 1;	// start neg. wave part
					if(posedge)		// if neg & pos & neg again -> end
						time=t;
					}
				}	
#else							/* PULSE */
			if(!time)			// time measurement one pulse
				{
				if(a<=VTRG)
					t++;
				
				if(a>VTRG)
					time=t;
				}	
#endif

			ptr++;
			if(ptr>39)
				ptr=0;
				 
			bitmask>>=1;		// shift bit mask right ones
        	}
    	}

#if(CURSOR1==ON && CURSOR2==ON)	// 2th Cursor on/off		
	if(time)
		{
		for (d=0 ;d<8 ;d++)			// set Cursor @ triggerpos
			graphbuf[8*((TRGPOS+time)/5)+d] ^= ( 0x10>>((TRGPOS+time)%5) );
		}
#endif

}

//----------------------------------------------------------------------------
void accquire(void)		// accquire 40Bytes analog stream / neg. slope
{
register u08 analog;

//	analog  =ADCL>>2;		// 
//	analog |=ADCH<<6;		// use only higher 8 bits
	analog  = inp(ADCL)>>2;		// 
	analog |= inp(ADCH)<<6;		// use only higher 8 bits

	sbi(ADCSR,6);
//	ADCSR |=0x40;	   	  	// start conversion for next accquire call

	abuf[aptr++]=analog;	// store level in buffer[pointer]

	if(aptr>39)				// count inpointer 0..39 and cycle
		aptr=0; 
	
	switch(phase)			// separate functions to accquire 
		{

		case 0 :	// wait for no trigger / be sure that signal is out off Trigger
			{
				if(analog>VTRG)	// no Trigger?
					{
					runtime=200;	// set time for untriggered mode 
					phase++;		// next -> show sync mode
					}
				else	// check timeout
					{
					runtime--;

					if(!runtime)	// no tigger, start stream if timeout
						{
						runtime = 1;	//
						phase = 2;	// next -> no trigger, show either unsync
						}
					}
			}; break;

		case 1 :	// wait for trigger
			{
				if(analog<=VTRG)	// Trigger?  neg. slope!
					{
					tptr = aptr;	// mind pointer
					runtime = 40-TRGPOS;	// record the rest
					phase++;		// next -> show sync mode
					}
				else	// check timeout
					{
					runtime--;

					if(!runtime)	// no tigger, start stream if timeout
						{
						runtime = 1;
						phase = 2;	// next -> no trigger, show either unsync
						}
					}
			}; break;

		case 2 :	// record 40byte analog stream
			{
				runtime--;
				if(!runtime)		// record done?
					{
					runtime = HOLDOFF;	// set holdoff time
					refresh = 1;	// Data ready for display
					phase++;		// next
					}
			}; break;
			 
		case 3 :	// holdoff
			{
				runtime--;
				if(!runtime)	// holdoff timeout?
					{
					runtime=200;
					phase=0;	// start again
					}
			}; break;
			 
		}
}

//----------------------------------------------------------------------------
// INTERRUPT

INTERRUPT(SIG_OVERFLOW0)	/* 1000Hz signal handler for tcnt0 overflow interrupt */
{
//	TCNT0=TMR0RELOAD;			// reset TCNT0 
	outp(TMR0RELOAD,TCNT0);		// reset TCNT0 

	if(!refresh)
		accquire();				// record analog stream
	
	if(delaytmr)				// public delay 1ms res.
		delaytmr--;

	if(0x08 & ~inp(PIND))	   	// check key at pin 3 port d 
		{
		keytmr--;

		if(keytmr<0)
			keytmr=1000;	   	// 1s repeat rate if pushed

			if(keytmr==1)
			{
			channel++;
			channel &=0x07;
			outp(channel,ADMUX);
			refresh = 1;
			time=0;
			phase=0;
			}
		}
	else
		keytmr=50;	   		   	// 50ms Key debounce
}

//----------------------------------------------------------------------------
// ms delay 1=1ms
void delay(u16 t)
{
	delaytmr=t;

	while(delaytmr);
}

//----------------------------------------------------------------------------

int main(void)
{
	outp(0,DDRA);	// input
	outp(0,PORTA);	// pullup's off
	cbi(DDRD,PD3);
	sbi(PORTD,PD3);

// initialize display, cursor off 
	lcd_init(LCD_DISP_ON);

// initialize ADC 
	outp(channel,ADMUX);
	outp(0xC2,ADCSR);	// 2MHz ADC CLOCK

// initialize Timer0 
    outp((1<<TOIE0), TIMSK); /* enable TCNT0 overflow */
	outp(TMR0RELOAD,TCNT0);		// reset TCNT0 
    outp(3, TCCR0); /* enable TCNT0 overflow */
	sei();                 // enable interrupts   

	lcd_puts_P("\f* Simple Scope *");
	delay(1000);
	lcd_puts_P("\fuser def. chars");	// defined in lcdfont.h 
	delay(1000);
	lcd_puts_P("\f\x80\x81\x82\x83\x84\x85\x86\x87 "); // display userdefined character rom 
	delay(1000);

	for (;;)
		{					// loop forever
		while(!refresh);	// new channel or new wave (not from fall of the '70)
		 
		drawgraph();		// draw graph in memory
		einer=time/10;
		zehntel=time-einer*10;
		
		lcd_setuserfont_r((u08 *)graphbuf);	// draw graph in LCD MEMORY->CGRAM
		refresh = 0;					// clear flag

   		lcd_puts_P("\n\x80\x81\x82\x83\x84\x85\x86\x87 "); // display userdefined character 
		if(einer)
			lcd_putc('0'+einer);
		else
			lcd_putc(' ');		// suppress leading zero 
		lcd_putc('0'+zehntel); 
		lcd_putc('m'); 
		lcd_putc('s'); 
		lcd_putc(' '); 
		lcd_putc('C'); 
		lcd_putc('1'+channel); 

		}

}
